<?php

/**
 * Plugin Name: UpLaunch
 * Plugin URI: https://www.uplaunch.com
 * Description: Create customized opt-in forms that add leads directly to your UpLaunch account.
 * Version: 2.0.6
 * Author: UpLaunch
 * Author URI: https://www.uplaunch.com
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Check for plugin updates via Kernl
 */
 
require_once 'plugin_update_check.php';

$KernlUpdater = new PluginUpdateChecker_2_0 (
    'https://kernl.us/api/v1/updates/5d00fffa54cbcb456a0bdc47/',
    __FILE__,
    'uplaunch',
    1
);

/**
 * Load styles and scripts
 */
 
add_action( 'admin_enqueue_scripts', 'uplaunch_admin_assets', 10, 1 );
function uplaunch_admin_assets($hook) {
	if( in_array($hook, array('post.php', 'post-new.php') ) ){
        $screen = get_current_screen();

        if ( is_object( $screen ) && 'uplaunch-form' == $screen->post_type ) {
			wp_enqueue_style( 'wp-color-picker');
			wp_enqueue_script( 'wp-color-picker');
			
			wp_enqueue_style( 'uplaunch-admin', plugin_dir_url( __FILE__ ) . 'css/admin.css', array(), '1.0.0');	 
			wp_enqueue_script( 'uplaunch-admin', plugin_dir_url( __FILE__ ) . 'js/admin.js', array( 'jquery'), '1.0.0', true );
		}
	}
}
 
add_action('wp_enqueue_scripts', 'uplaunch_frontend_assets');
function uplaunch_frontend_assets() {
	wp_register_style( 'uplaunch-style', plugin_dir_url( __FILE__ ) . 'css/uplaunch.css', array(), '1.0.0');	 
	wp_register_script( 'uplaunch-script', plugin_dir_url( __FILE__ ) . 'js/uplaunch.js', array( 'jquery'), '1.0.0', true );
}

/**
 * Create UpLaunch Settings page in WordPress admin
 */

add_action('admin_menu', 'uplaunch_settings_page');
function uplaunch_settings_page() {	
	add_submenu_page( 
		'edit.php?post_type=uplaunch-form', 
		'UpLaunch Settings', 
		'Settings', 
		'manage_options', 
		'uplaunch-settings', 
		'uplaunch_settings' 
	);

	add_action( 'admin_init', 'uplaunch_register_settings' );
}

function uplaunch_register_settings() {
	register_setting( 'uplaunch-settings-group', 'uplaunch_api_key' );
}

function uplaunch_settings() {
	include_once( 'uplaunch-settings-page.php' );
}

/**
 * Show admin warning if there is a problem with the API key
 */

add_action( 'admin_notices', 'uplaunch_admin_notice' );
function uplaunch_admin_notice() {
    $screen = get_current_screen();
	$settings_link = admin_url().'edit.php?post_type=uplaunch-form&page=uplaunch-settings';

	if ( is_object( $screen ) && 'uplaunch-form' == $screen->post_type ) {
		if ( $screen->base != 'uplaunch-form_page_uplaunch-settings') {
			$api_key = get_option('uplaunch_api_key');
			$auth = uplaunch_check_authentication();
			
			if ( $api_key ) {
				if ( ! isset($auth->account_name) ) {
					echo '<div class="notice notice-error"><p>There seems to be a problem with your UpLaunch API Key that may prevent your leads from being captured. Please check your settings <a href="'.$settings_link.'">here</a>.</p></div>';
				}
			}
			else {
				echo '<div class="notice"><p>To begin using UpLaunch, please enter your API Key <a href="'.$settings_link.'">here</a>.</p></div>';
			}
		}
	}
}

/**
 * Create UpLaunch Form custom post type
 */
 
add_action( 'init', 'uplaunch_form_post_type' );
function uplaunch_form_post_type() {
    $args = array(
		'description' => 'UpLaunch Form Post Type',
		'show_ui' => true,
		'exclude_from_search' => true,
		'menu_position' => 2,
		'menu_icon' => 'data:image/svg+xml;base64, '.base64_encode('<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path fill="#a0a5aa" d="M505.05 19.1a15.89 15.89 0 0 0-12.2-12.2C460.65 0 435.46 0 410.36 0c-103.2 0-165.1 55.2-211.29 128H94.87A48 48 0 0 0 52 154.49l-49.42 98.8A24 24 0 0 0 24.07 288h103.77l-22.47 22.47a32 32 0 0 0 0 45.25l50.9 50.91a32 32 0 0 0 45.26 0L224 384.16V488a24 24 0 0 0 34.7 21.49l98.7-49.39a47.91 47.91 0 0 0 26.5-42.9V312.79c72.59-46.3 128-108.4 128-211.09.1-25.2.1-50.4-6.85-82.6zM384 168a40 40 0 1 1 40-40 40 40 0 0 1-40 40z"/></svg>'),
		'labels' => array(
			'menu_name' => 'UpLaunch',
			'name' => 'UpLaunch Forms',
			'all_items' => 'View Forms',
			'singular_name' => 'Form',
			'add_new' => 'Add New Form',
			'add_new_item' => 'Add New Form',
			'edit' => 'Edit Forms',
			'edit_item' => 'Edit Form',
			'new-item' => 'New Form',
			'view' => 'View Forms',
			'view_item' => 'View Form',
			'search_items' => 'Search Forms',
			'not_found' => 'No Forms Found',
			'not_found_in_trash' => 'No Forms Found in Trash',
			'parent' => 'Parent Form'				   
		),
		'public' => false,
		'has_archive' => false,
		'capability_type' => 'post',
		'hierarchical' => false,
		'rewrite' => true, 
		'supports' => array('title')
	);
	
    register_post_type( 'uplaunch-form' , $args );
}

/**
 * UpLaunch Forms custom meta fields 
 */

add_action('add_meta_boxes', 'uplaunch_form_meta_boxes');
function uplaunch_form_meta_boxes() {
	add_meta_box(
		'uplaunch_shortcode',
		'Shortcode',  
		'uplaunch_the_shortcode_reference',
		'uplaunch-form',
		'advanced',
		'high'
	);
	
	add_meta_box(
		'uplaunch_form_details',
		'Form Details',  
		'uplaunch_form_details_html',
		'uplaunch-form'
	);
}

function uplaunch_form_details_html($post) {
	//Nonce
	?>
	 	<input type="hidden" name="uplaunch_nonce" value="<?php echo wp_create_nonce( basename(__FILE__) ); ?>">
	<?php
	
	// Journey
	$value = get_post_meta($post->ID, '_journey', true);
	?>
	<p>
		<label for="journey">Incoming Journey</label><br>
		<select id="journey" name="journey">
			<?php echo uplaunch_get_journey_options($value); ?>
		</select>
	</p>
	<?php
	
	// Ask for Number
	$value = get_post_meta($post->ID, '_ask_for_number', true);
	?>
	<p>
		<label for="ask-for-number">Ask for Phone Number?</label><br>
		<select id="ask-for-number" name="ask_for_number">
			<option value="required" <?php selected($value, 'required'); ?>>Yes - Required</option>
			<option value="optional" <?php selected($value, 'optional'); ?>>Yes - Optional</option>
			<option value="no" <?php selected($value, 'no'); ?>>No</option>
		</select>
	</p>
	<?php

	// Button Text
	$value = get_post_meta($post->ID, '_button_text', true);
	$value = ( $value ) ?: 'Submit';
    ?>
	<p>
		<label for="button-text">Button Text</label><br>
		<input id="button-text" name="button_text" type="text" value="<?php echo $value; ?>" />
	</p>
    <?php
	
	// Button Background
	$value = get_post_meta($post->ID, '_button_background', true);
	$value = ( $value ) ?: '#7b8794';
    ?>
	<p>
		<label for="button-background">Button Background</label><br>
		<input id="button-background" name="button_background" class="color-picker" type="text" value="<?php echo $value; ?>" />
	</p>
    <?php
	
	// Button Text Color
	$value = get_post_meta($post->ID, '_button_text_color', true);
	$value = ( $value ) ?: '#ffffff';
    ?>
	<p>
		<label for="button-text-color">Button Text Color</label><br>
		<input id="button-text-color" name="button_text_color" class="color-picker" type="text" value="<?php echo $value; ?>" />
	</p>
	<?php
}

add_action('save_post', 'uplaunch_form_save_custom_fields');
function uplaunch_form_save_custom_fields($post_id) {
	if ( !empty($_POST) ) {
		// Check nonce
		if ( ! wp_verify_nonce( $_POST['uplaunch_nonce'], basename(__FILE__) ) ) {
			return $post_id;
		}
		
		// Check autosave
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return $post_id;
		}
		
		// Check user permission
		if ( 'uplaunch-form' === $_POST['post_type'] ) {
			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return $post_id;
			}
		}
		
		// Save fields
		if ( array_key_exists('journey', $_POST) ) {
			update_post_meta(
				$post_id,
				'_journey',
				sanitize_text_field($_POST['journey'])
			);
		}
		if ( array_key_exists('ask_for_number', $_POST) ) {
			update_post_meta(
				$post_id,
				'_ask_for_number',
				sanitize_text_field($_POST['ask_for_number'])
			);
		}
		if ( array_key_exists('button_text', $_POST) ) {
			update_post_meta(
				$post_id,
				'_button_text',
				sanitize_text_field($_POST['button_text'])
			);
		}
		if ( array_key_exists('button_background', $_POST) ) {
			update_post_meta(
				$post_id,
				'_button_background',
				sanitize_hex_color($_POST['button_background'])
			);
		}	
		if ( array_key_exists('button_text_color', $_POST) ) {
			update_post_meta(
				$post_id,
				'_button_text_color',
				sanitize_hex_color($_POST['button_text_color'])
			);
		}
	}
}

/**
 * UpLaunch Forms custom columns 
 */

add_action('manage_edit-uplaunch-form_columns', 'uplaunch_form_columns');
function uplaunch_form_columns($columns) {
	$columns['uplaunch-form-journey'] = "Journey";
	$columns['uplaunch-form-shortcode'] = "Shortcode";
	unset($columns['date']);
	
	return $columns;
}

add_action('manage_uplaunch-form_posts_custom_column','uplaunch_form_column_content');
function uplaunch_form_column_content($column_name){
	global $post;
	
	switch ($column_name) {	 
		case 'uplaunch-form-journey' :	
			$journey = get_post_meta($post->ID, '_journey', true);
			$journey = explode(':', $journey);
			echo ( isset($journey[1]) ) ? $journey[1] : '';
			break;
		case 'uplaunch-form-shortcode' :	
			uplaunch_the_shortcode_reference($post);
			break;
		default :
			break;
    }
}


/**
 * UpLaunch Shortcode
 */

add_shortcode('uplaunch', 'uplaunch_render_shortcode');
function uplaunch_render_shortcode($atts) {
	// Avoid rendering this shortcode output within the Elementor Page Builder Editor
	$current_page_url = 'http://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
	if (strpos($current_page_url, 'elementor') !== false) {
		return;
	}
	
	// Make sure we have a valid form and API connection
	$error = false;
	$api_key = get_option('uplaunch_api_key');
	$form_id = $atts['form'];
	$form = uplaunch_maybe_get_form_post($form_id);
	
	if ( ! $api_key ) {
		$error = true;
	}
	else {
		$auth = uplaunch_check_authentication();
		if ( ! isset($auth->account_name) ) {
			$error = true;
		}
	}
	
	if ( ! is_object($form) || $form->post_status != 'publish' || $form->post_type != 'uplaunch-form' ) {
		$error = true;
	}
	
	if ( $error ) {
		return '<p>Sorry, this UpLaunch shortcode is not valid. Please check your UpLaunch settings.</p>';
	}
	
	$journey = get_post_meta($form->ID, '_journey', true);
	$journey = explode(':', $journey);
	
	$ask_for_number = get_post_meta($form->ID, '_ask_for_number', true);
	$button_text = get_post_meta($form->ID, '_button_text', true);
	$button_background = get_post_meta($form->ID, '_button_background', true);
	$button_text_color = get_post_meta($form->ID, '_button_text_color', true);

	$sizing = ( $ask_for_number == 'no' ) ? 'twelve' : 'six';
	
	ob_start();
	
	?>
	
	<div class="uplaunch_container">
		<div class="uplaunch_first_name uplaunch_input">
			<input type="text" name="first_name" placeholder="First name">
		</div>
		<div class="uplaunch_last_name uplaunch_input">
			<input type="text" name="last_name" placeholder="Last name">
		</div>
		<div class="uplaunch_email uplaunch_input <?php echo $sizing; ?>">
			<input type="text" name="email" placeholder="Email">
			<input type="hidden" name="journey_id" value="<?php echo $journey[0]; ?>" />
			<input type="hidden" name="journey_name" value="<?php echo $journey[1]; ?>" />
			<input type="hidden" name="api_key" value="<?php echo $api_key; ?>" />
		</div>
		
		<?php if ( $ask_for_number == 'required' ) : ?>
			<div class="uplaunch_phone uplaunch_input <?php echo $sizing; ?>">
				<input type="text" name="phone" placeholder="Phone" required>
			</div>
		<?php elseif ( $ask_for_number == 'optional' ) : ?>
			<div class="uplaunch_phone uplaunch_input <?php echo $sizing; ?>">
				<input type="text" name="phone" placeholder="Phone (optional)">
			</div>
		<?php else : ?>
			<input type="hidden" name="phone" value=""/>
		<?php endif; ?>
		
		<button class="uplaunch_button" style="background-color: <?php echo $button_background; ?>;color: <?php echo $button_text_color; ?>;"><?php echo $button_text; ?></button>	
	</div>
		
	<?php
	
	wp_enqueue_style( 'uplaunch-style' );
	wp_enqueue_script( 'uplaunch-script' );
	
	// Strip whitespace and line breaks above to preserve styling of inline elements
	return preg_replace('~>\s*\n\s*<~', '><', ob_get_clean());
}

/**
 *  Transition data prior to version 2.0.0
 */

add_action('admin_init', 'uplaunch_transition_to_version_2');
function uplaunch_transition_to_version_2() {
	$uplaunch_path = ABSPATH.'uplaunch';
	$data_path = $uplaunch_path.'/data';
	$forms_path = $data_path.'/forms';
	$settings_path = $data_path.'/settings.json';
	
	if ( ! is_dir($uplaunch_path) ) {
		return;
	}
	
	if ( file_exists($settings_path) ) {
		$settings = @json_decode(file_get_contents($settings_path), 1);
		if ( isset($settings['api_key']) ) {
			update_option('uplaunch_api_key', $settings['api_key']);
		}
		
		unlink($settings_path);
	}
	
	$scan = @scandir($forms_path);
	foreach ($scan as $file) {
		$form_file_path = $forms_path.'/'.$file; 
		if ( is_file($form_file_path) ) {
			$form_data = json_decode(file_get_contents($form_file_path), 1);
			$legacy_form_id = str_replace('.json', '', $file);
			
			$args = array(
				'post_type' => 'uplaunch-form',
				'post_status' => 'publish',
				'post_title' => $form_data['form_name'],
				'meta_input' => array(
					'_journey' => $form_data['journey'],
					'_ask_for_number' => $form_data['ask_for_number'],
					'_button_text' => $form_data['button_text'],
					'_button_background' => $form_data['button_background'],
					'_button_text_color' => $form_data['button_text_color'],
					'_uplaunch_legacy_id' => $legacy_form_id,
				),
			);
			
			if ( wp_insert_post($args) ) {
				unlink($form_file_path);
			}	
		}
	}
	
	if ( is_dir($forms_path) ) rmdir($forms_path);
	if ( is_dir($data_path) ) rmdir($data_path);
	if ( is_dir($uplaunch_path) ) rmdir($uplaunch_path);
}

/**
 *  Helper functions
 */
 
function uplaunch_check_authentication() {
	$api_key = get_option('uplaunch_api_key');

	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, 'https://api.uplaunch.com/api/v1/auth');
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		'Authorization: '.$api_key
		));
	$data = curl_exec($ch);
	curl_close($ch);

	return json_decode($data);
}

function uplaunch_get_journey_options($current_journey) {
	$api_key = get_option('uplaunch_api_key');
	$output = '';
	
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, 'https://api.uplaunch.com/api/v1/journeys');
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		'Authorization: '.$api_key
		));
	$data = curl_exec($ch);
	curl_close($ch);

	$journeys = json_decode($data);
	if ( $journeys ) {
		foreach ($journeys as $journey) {
			$key = $journey->id.':'.$journey->name;
			$output .= '<option value="'.$key.'" '.selected($key, $current_journey).'>'.$journey->name.'</option>';
		}
	}

	return $output;	
}

function uplaunch_maybe_get_form_post($form_id) {
	global $wpdb;
	
	$query = "SELECT post_id FROM ".$wpdb->postmeta."
	          WHERE meta_key = '_uplaunch_legacy_id'
			  AND meta_value = '".$form_id."'";		  
	$legacy_form_id = $wpdb->get_var($query);
	
	if ( $legacy_form_id ) {
		$form = get_post($legacy_form_id);
		
		return $form;
	}
	else {	
		$form = get_post($form_id);
		
		return $form;
	}	
}

function uplaunch_the_shortcode_reference($post) {
	echo '[uplaunch form="'.$post->ID.'"]';
}
